@@ -1,3 +1,4 @@
+use crate::ui::components::Keys;
 use crate::ui::components::StyleColorSymbol;
 use crate::ui::components::Xywh;
 use crate::ui::Loop;
@@ -28,7 +29,6 @@ use anyhow::{anyhow, Result};
 use serde::{Deserialize, Serialize};
 use std::fs::{self, read_to_string};
 use std::path::PathBuf;
-use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 
 pub const MUSIC_DIR: &str = "~/Music";
 
@@ -42,46 +42,11 @@ pub struct Termusic {
     pub add_playlist_front: bool,
     pub disable_exit_confirmation: bool,
     pub theme_selected: String,
-    pub style_color_symbol: StyleColorSymbol,
     pub album_photo_xywh: Xywh,
+    pub style_color_symbol: StyleColorSymbol,
     pub keys: Keys,
 }
 
-#[derive(Clone, Deserialize, Serialize)]
-pub struct Keys {
-    pub quit: KeyEvent,
-    pub vim_h: KeyEvent,
-    pub vim_j: KeyEvent,
-    pub vim_k: KeyEvent,
-    pub vim_l: KeyEvent,
-}
-
-impl Default for Keys {
-    fn default() -> Self {
-        Self {
-            quit: KeyEvent {
-                code: Key::Char('q'),
-                modifiers: KeyModifiers::NONE,
-            },
-            vim_h: KeyEvent {
-                code: Key::Char('h'),
-                modifiers: KeyModifiers::NONE,
-            },
-            vim_j: KeyEvent {
-                code: Key::Char('j'),
-                modifiers: KeyModifiers::NONE,
-            },
-            vim_k: KeyEvent {
-                code: Key::Char('k'),
-                modifiers: KeyModifiers::NONE,
-            },
-            vim_l: KeyEvent {
-                code: Key::Char('l'),
-                modifiers: KeyModifiers::NONE,
-            },
-        }
-    }
-}
 impl Default for Termusic {
     fn default() -> Self {
         Self {
