@@ -55,6 +55,7 @@ pub enum Msg {
     GeneralSearch(GSMsg),
     HelpPopupShow,
     HelpPopupClose,
+    KeyEditor(KEMsg),
     Library(LIMsg),
     LyricCycle,
     LyricAdjustDelay(i64),
@@ -145,7 +146,7 @@ pub enum TEMsg {
     TETextareaLyricBlurDown,
     TETextareaLyricBlurUp,
 }
-// #[derive(Debug, Clone, PartialEq)]
+
 #[derive(Clone, Debug, PartialEq)]
 pub enum CEMsg {
     ColorChanged(IdColorEditor, ColorConfig),
@@ -193,6 +194,154 @@ pub enum CEMsg {
     ThemeSelectBlurUp,
     ThemeSelectLoad(usize),
 }
+
+#[derive(Clone, Debug, PartialEq)]
+pub enum KEMsg {
+    GlobalColorEditorBlurDown,
+    GlobalColorEditorBlurUp,
+    GlobalColorEditorInputBlurDown,
+    GlobalColorEditorInputBlurUp,
+    GlobalDownBlurDown,
+    GlobalDownBlurUp,
+    GlobalDownInputBlurDown,
+    GlobalDownInputBlurUp,
+    GlobalGotoBottomBlurDown,
+    GlobalGotoBottomBlurUp,
+    GlobalGotoBottomInputBlurDown,
+    GlobalGotoBottomInputBlurUp,
+    GlobalGotoTopBlurDown,
+    GlobalGotoTopBlurUp,
+    GlobalGotoTopInputBlurDown,
+    GlobalGotoTopInputBlurUp,
+    GlobalHelpBlurDown,
+    GlobalHelpBlurUp,
+    GlobalHelpInputBlurDown,
+    GlobalHelpInputBlurUp,
+    GlobalKeyEditorBlurDown,
+    GlobalKeyEditorBlurUp,
+    GlobalKeyEditorInputBlurDown,
+    GlobalKeyEditorInputBlurUp,
+    GlobalLeftBlurDown,
+    GlobalLeftBlurUp,
+    GlobalLeftInputBlurDown,
+    GlobalLeftInputBlurUp,
+    GlobalLyricAdjustForwardBlurDown,
+    GlobalLyricAdjustForwardBlurUp,
+    GlobalLyricAdjustBackwardBlurDown,
+    GlobalLyricAdjustBackwardBlurUp,
+    GlobalLyricAdjustForwardInputBlurDown,
+    GlobalLyricAdjustForwardInputBlurUp,
+    GlobalLyricAdjustBackwardInputBlurDown,
+    GlobalLyricAdjustBackwardInputBlurUp,
+    GlobalLyricCyleBlurDown,
+    GlobalLyricCyleBlurUp,
+    GlobalLyricCyleInputBlurDown,
+    GlobalLyricCyleInputBlurUp,
+    GlobalPlayerNextBlurDown,
+    GlobalPlayerNextBlurUp,
+    GlobalPlayerNextInputBlurDown,
+    GlobalPlayerNextInputBlurUp,
+    GlobalPlayerPreviousBlurDown,
+    GlobalPlayerPreviousBlurUp,
+    GlobalPlayerPreviousInputBlurDown,
+    GlobalPlayerPreviousInputBlurUp,
+    GlobalPlayerSeekForwardBlurDown,
+    GlobalPlayerSeekForwardBlurUp,
+    GlobalPlayerSeekForwardInputBlurDown,
+    GlobalPlayerSeekForwardInputBlurUp,
+    GlobalPlayerSeekBackwardBlurDown,
+    GlobalPlayerSeekBackwardBlurUp,
+    GlobalPlayerSeekBackwardInputBlurDown,
+    GlobalPlayerSeekBackwardInputBlurUp,
+    GlobalPlayerTogglePauseBlurDown,
+    GlobalPlayerTogglePauseBlurUp,
+    GlobalPlayerTogglePauseInputBlurDown,
+    GlobalPlayerTogglePauseInputBlurUp,
+    GlobalQuitBlurDown,
+    GlobalQuitBlurUp,
+    GlobalQuitInputBlurDown,
+    GlobalQuitInputBlurUp,
+    GlobalRightBlurDown,
+    GlobalRightBlurUp,
+    GlobalRightInputBlurDown,
+    GlobalRightInputBlurUp,
+    GlobalUpBlurDown,
+    GlobalUpBlurUp,
+    GlobalUpInputBlurDown,
+    GlobalUpInputBlurUp,
+    GlobalVolumeDownBlurDown,
+    GlobalVolumeDownBlurUp,
+    GlobalVolumeDownInputBlurDown,
+    GlobalVolumeDownInputBlurUp,
+    GlobalVolumeUpBlurDown,
+    GlobalVolumeUpBlurUp,
+    GlobalVolumeUpInputBlurDown,
+    GlobalVolumeUpInputBlurUp,
+    HelpPopupShow,
+    HelpPopupClose,
+    KeyChanged(IdKeyEditor),
+    KeyEditorShow,
+    KeyEditorCloseCancel,
+    KeyEditorCloseOk,
+    LibraryDeleteBlurDown,
+    LibraryDeleteBlurUp,
+    LibraryDeleteInputBlurDown,
+    LibraryDeleteInputBlurUp,
+    LibraryLoadDirBlurDown,
+    LibraryLoadDirBlurUp,
+    LibraryLoadDirInputBlurDown,
+    LibraryLoadDirInputBlurUp,
+    LibraryPasteBlurDown,
+    LibraryPasteBlurUp,
+    LibraryPasteInputBlurDown,
+    LibraryPasteInputBlurUp,
+    LibrarySearchBlurDown,
+    LibrarySearchBlurUp,
+    LibrarySearchInputBlurDown,
+    LibrarySearchInputBlurUp,
+    LibrarySearchYoutubeBlurDown,
+    LibrarySearchYoutubeBlurUp,
+    LibrarySearchYoutubeInputBlurDown,
+    LibrarySearchYoutubeInputBlurUp,
+    LibraryTagEditorBlurDown,
+    LibraryTagEditorBlurUp,
+    LibraryTagEditorInputBlurDown,
+    LibraryTagEditorInputBlurUp,
+    LibraryYankBlurDown,
+    LibraryYankBlurUp,
+    LibraryYankInputBlurDown,
+    LibraryYankInputBlurUp,
+    PlaylistDeleteBlurDown,
+    PlaylistDeleteBlurUp,
+    PlaylistDeleteInputBlurDown,
+    PlaylistDeleteInputBlurUp,
+    PlaylistDeleteAllBlurDown,
+    PlaylistDeleteAllBlurUp,
+    PlaylistDeleteAllInputBlurDown,
+    PlaylistDeleteAllInputBlurUp,
+    PlaylistShuffleBlurDown,
+    PlaylistShuffleBlurUp,
+    PlaylistShuffleInputBlurDown,
+    PlaylistShuffleInputBlurUp,
+    PlaylistModeCycleBlurDown,
+    PlaylistModeCycleBlurUp,
+    PlaylistModeCycleInputBlurDown,
+    PlaylistModeCycleInputBlurUp,
+    PlaylistPlaySelectedBlurDown,
+    PlaylistPlaySelectedBlurUp,
+    PlaylistPlaySelectedInputBlurDown,
+    PlaylistPlaySelectedInputBlurUp,
+    PlaylistAddFrontBlurDown,
+    PlaylistAddFrontBlurUp,
+    PlaylistAddFrontInputBlurDown,
+    PlaylistAddFrontInputBlurUp,
+    PlaylistSearchBlurDown,
+    PlaylistSearchBlurUp,
+    PlaylistSearchInputBlurDown,
+    PlaylistSearchInputBlurUp,
+    RadioOkBlurUp,
+    RadioOkBlurDown,
+}
 // Let's define the component ids for our application
 #[derive(Debug, Eq, PartialEq, Clone, Hash)]
 pub enum Id {
@@ -200,12 +349,13 @@ pub enum Id {
     DeleteConfirmRadioPopup,
     DeleteConfirmInputPopup,
     ErrorPopup,
-    GlobalListener,
-    HelpPopup,
-    Label,
-    Library,
     GeneralSearchInput,
     GeneralSearchTable,
+    GlobalListener,
+    HelpPopup,
+    KeyEditor(IdKeyEditor),
+    Label,
+    Library,
     Lyric,
     MessagePopup,
     Playlist,
@@ -256,6 +406,81 @@ pub enum IdColorEditor {
     LyricBorder,
 }
 
+#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+pub enum IdKeyEditor {
+    GlobalColorEditor,
+    GlobalColorEditorInput,
+    GlobalDown,
+    GlobalDownInput,
+    GlobalGotoBottom,
+    GlobalGotoBottomInput,
+    GlobalGotoTop,
+    GlobalGotoTopInput,
+    GlobalHelp,
+    GlobalHelpInput,
+    GlobalKeyEditor,
+    GlobalKeyEditorInput,
+    GlobalLeft,
+    GlobalLeftInput,
+    GlobalLyricAdjustForward,
+    GlobalLyricAdjustForwardInput,
+    GlobalLyricAdjustBackward,
+    GlobalLyricAdjustBackwardInput,
+    GlobalLyricCycle,
+    GlobalLyricCycleInput,
+    GlobalPlayerTogglePause,
+    GlobalPlayerTogglePauseInput,
+    GlobalPlayerNext,
+    GlobalPlayerNextInput,
+    GlobalPlayerPrevious,
+    GlobalPlayerPreviousInput,
+    GlobalPlayerSeekForward,
+    GlobalPlayerSeekForwardInput,
+    GlobalPlayerSeekBackward,
+    GlobalPlayerSeekBackwardInput,
+    GlobalQuit,
+    GlobalQuitInput,
+    GlobalRight,
+    GlobalRightInput,
+    GlobalUp,
+    GlobalUpInput,
+    GlobalVolumeDown,
+    GlobalVolumeDownInput,
+    GlobalVolumeUp,
+    GlobalVolumeUpInput,
+    HelpPopup,
+    LabelHint,
+    LibraryDelete,
+    LibraryDeleteInput,
+    LibraryLoadDir,
+    LibraryLoadDirInput,
+    LibraryPaste,
+    LibraryPasteInput,
+    LibrarySearch,
+    LibrarySearchInput,
+    LibrarySearchYoutube,
+    LibrarySearchYoutubeInput,
+    LibraryTagEditor,
+    LibraryTagEditorInput,
+    LibraryYank,
+    LibraryYankInput,
+    PlaylistDelete,
+    PlaylistDeleteAll,
+    PlaylistShuffle,
+    PlaylistModeCycle,
+    PlaylistPlaySelected,
+    PlaylistAddFront,
+    PlaylistSearch,
+    PlaylistDeleteInput,
+    PlaylistDeleteAllInput,
+    PlaylistShuffleInput,
+    PlaylistModeCycleInput,
+    PlaylistPlaySelectedInput,
+    PlaylistAddFrontInput,
+    PlaylistSearchInput,
+    RadioOk,
+}
+
 #[derive(Clone, Copy)]
 pub enum Status {
     Running,
